/*--------------------------------------------------------------------------

   MOM files displayer
   events.c v1.0 7/March/2001
   Copyright (C) 2001 J. Ribelles

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

  --------------------------------------------------------------------------*/


#include <stdlib.h>
#include <GL/glut.h>
#include "minVis.h"


// Variables
short  mouseButtonPressed= NO_BUTTON_PRESSED;  // what button is pressed??
unsigned short  oldMouseX= 0, oldMouseY= 0;    // previous mouse coordinates
float angleX, angleY;                          // X and Y rotation angles
int ctrlRotation= 0;                           // to rotate the model or not


/*--------------------------------------------------------------------------
   Proyection...
  --------------------------------------------------------------------------*/
void proyection (void) {

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(fovy, aspect, 1.0f, 10.0f);
}


/*--------------------------------------------------------------------------
   Reshape...
  --------------------------------------------------------------------------*/
void reshape (int width, int height) {
  
  glViewport (0, 0, width, height);
  aspect= (GLdouble) width/ (GLdouble) height;
  proyection();
}


/*--------------------------------------------------------------------------
   Keyboard...
  --------------------------------------------------------------------------*/
void keyboard (unsigned char key, int px, int py) {

  switch (key) {
  case  27: exit (0); break;
  case 'q': exit (0); break;
  case 'Q': exit (0); break;
  }
}


/*--------------------------------------------------------------------------
   Mouse...
  --------------------------------------------------------------------------*/
void mouse (int button, int state, int x, int y) {
 
  switch (button) {
  case GLUT_LEFT_BUTTON: {
    if (state== GLUT_DOWN)
      if ((glutGetModifiers()== 1) && (modeLOD== 1))
	mouseButtonPressed= GLUT_LEFT_BUTTON_AND_SHIFT;
      else if (glutGetModifiers()== 2){
	mouseButtonPressed= GLUT_LEFT_BUTTON_AND_CONTROL;
      }
      else {
	mouseButtonPressed= GLUT_LEFT_BUTTON;
	ctrlRotation= 0;
      }
    else {
      mouseButtonPressed= NO_BUTTON_PRESSED;
      if (ctrlRotation) { angleX= 0; angleY= 0;}
      glutPostRedisplay ();
    }
    break;
  }
  }
} 


/*--------------------------------------------------------------------------
   Mouse motion
  --------------------------------------------------------------------------*/
void motion (int newMouseX, int newMouseY) {

  switch (mouseButtonPressed) {
    case GLUT_LEFT_BUTTON: {
      mouseButtonPressed= LEFT_BUTTON_HOLDED;
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      break;
    }
    case GLUT_LEFT_BUTTON_AND_SHIFT: {
      mouseButtonPressed= LEFT_BUTTON_AND_SHIFT_HOLDED;
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      break;
    }
    case GLUT_LEFT_BUTTON_AND_CONTROL: {
      mouseButtonPressed= LEFT_BUTTON_AND_CONTROL_HOLDED;
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      break;
    }
    case LEFT_BUTTON_HOLDED: {  // rotation
      angleX= (float)(newMouseX- oldMouseX)/ 2.0f;
      angleY= (float)(newMouseY- oldMouseY)/ 2.0f;
      glMatrixMode (GL_MODELVIEW);
      glLoadIdentity ();
      glRotatef (angleX, 0.0f,1.0f,0.0f);
      glRotatef (angleY, 1.0f,0.0f,0.0f);
      glMultMatrixf (matrixRotation);
      glGetFloatv (GL_MODELVIEW_MATRIX, matrixRotation);
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      ctrlRotation= 0;
      glutPostRedisplay ();
      break;
    }
    case LEFT_BUTTON_AND_CONTROL_HOLDED: {  // zoom
      fovy-= (newMouseY- oldMouseY)/ 3.0f;
      if (fovy> 140.0f) fovy= 140.0f;
      else if (fovy< 1.0f) fovy= 1.0f;
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      proyection();
      glutPostRedisplay ();
      break;
    }
    case LEFT_BUTTON_AND_SHIFT_HOLDED: { // changing the LOD
      Lod-= (newMouseY- oldMouseY)* 5;
      if (Lod>= NLods) Lod= NLods- 1;
      else if (Lod< 0) Lod= 0;
      oldMouseX= newMouseX; 
      oldMouseY= newMouseY;
      proyection();
      glutPostRedisplay ();
      break;
    }
  }
}


/*--------------------------------------------------------------------------
   Idle state
  --------------------------------------------------------------------------*/
void idle (void) {

  if ((mouseButtonPressed!= GLUT_LEFT_BUTTON) && 
      (mouseButtonPressed!= LEFT_BUTTON_HOLDED)) {
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    glRotatef (angleX, 0.0f,1.0f,0.0f);
    glRotatef (angleY, 1.0f,0.0f,0.0f);
    glMultMatrixf (matrixRotation);
    glGetFloatv (GL_MODELVIEW_MATRIX, matrixRotation);
    glutPostRedisplay ();
  } else 
    ctrlRotation= 1;
}


/*--------------------------------------------------------------------------
   End of events.cpp
  --------------------------------------------------------------------------*/
