/*--------------------------------------------------------------------------

   MOM files displayer
   visMom.cpp v1.0 7/March/2001
   Copyright (C) 2001 J. Ribelles

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

  --------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------
   C++ includes
  --------------------------------------------------------------------------*/
#include <math.h>
#include <GL/glut.h>
#include <time.h>
#include <stdio.h>


/*--------------------------------------------------------------------------
   Application includes
  --------------------------------------------------------------------------*/
#include "Mom.h"
#include "minVis.h"


// Variables
float   matrixRotation[16]=   { 1.0f, 0.0, 0.0f, 0.0f,  // rotation
                                0.0f, 1.0, 0.0f, 0.0f,
                                0.0f, 0.0, 1.0f, 0.0f,
                                0.0f, 0.0, 0.0f, 1.0f};
GLdouble  aspect;                 // aspect ratio  of the window
float     fovy= 30.0f;            // camera fovy
Mom       model;                  // the mom model
int       Lod= 0,                 // the LOD visualized 
          NLods= 0,               // the number of LODS 
          nFrames= 0,             // the number of frames visualized
          lastCountFrames= 0;     // the frames visualized in the last second


/*--------------------------------------------------------------------------
   The frame rate
  --------------------------------------------------------------------------*/
void frameRate (int facesPainted) {
  
  char Info[50], *tmp;
  int  scale= 2000;

  glDisable        (GL_DEPTH_TEST);

  glMatrixMode (GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity ();
  glOrtho(-1.0*aspect*scale, 1.0*aspect*scale, 
	  -1.0*scale, 1.0*scale, -1.0, 1.0);

  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  glTranslatef (-scale/1.5, -scale/1.1, 0.0f);

  sprintf (Info, "LOD: %3.2f - Faces: %4d - Fps: %4d\n",
	   1.0f- ((float)Lod/NLods), facesPainted, lastCountFrames);

  tmp=Info;

  glColor3f (1.0f, 1.0f, 0.5f);
  while (*tmp != '\n') {
    glutStrokeCharacter (GLUT_STROKE_ROMAN, *tmp);
    tmp++;
  }

  glMatrixMode (GL_PROJECTION);
  glPopMatrix();

  glEnable (GL_DEPTH_TEST);
}


/*--------------------------------------------------------------------------
   Count the frames per second visualized
  --------------------------------------------------------------------------*/
void theTimer (int value) {

  lastCountFrames= nFrames;
  nFrames= 0;
  glutTimerFunc (1000, theTimer, 0);
}


/*--------------------------------------------------------------------------
   Drawing function
  --------------------------------------------------------------------------*/
void  display(void) {
  
  int facesPainted;

  glClearColor (0.24f, 0.47f, 0.60f, 1.0f);
  glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

  // transformation of the model
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  glTranslatef (0.0f, 0.0f, -5.0f);
  glMultMatrixf (matrixRotation);

  // compute the LOD depending on the camera fovy
  if (modeLOD== 2)
    if (fovy== 1.0) Lod= 0;
    else if (fovy== 140) Lod= NLods-1;
    else Lod= (int)(NLods* (float)(1.0f- (pow (5001.0f, (140.0f-(float)fovy)/139.0f)- 1.0f)* 0.0002f));
  
  // drawing the object
  glColor3f (.95f, .95f, .95f);
  glBegin (GL_TRIANGLES);
  facesPainted= model.displayLod(Lod);
  glEnd();

  nFrames++;
  frameRate(facesPainted);

  // switch the buffer
  glutSwapBuffers ();
}


/*--------------------------------------------------------------------------
   Render options...
  --------------------------------------------------------------------------*/
void initRenderOptions (void) {

  glPolygonMode   (GL_FRONT_AND_BACK, GL_LINE);
  glFrontFace     (GL_CCW);
  glCullFace      (GL_BACK);
  glDisable       (GL_CULL_FACE);
  glShadeModel    (GL_FLAT);
  glDepthFunc     (GL_LEQUAL);
  glEnable        (GL_DEPTH_TEST);
}


/*--------------------------------------------------------------------------
   View...
  --------------------------------------------------------------------------*/
void initView (void) {

  glMatrixMode   (GL_MODELVIEW);
  glLoadIdentity ();
}


/*--------------------------------------------------------------------------
   Displaying the application window
  --------------------------------------------------------------------------*/
void initWindow (int argc, char **argv) {

  glutInit(&argc, argv);
  glutInitDisplayMode    (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize     (mainWindowWidth, mainWindowHeight);
  glutInitWindowPosition (mainWindowPosX, mainWindowPosY);
  glutCreateWindow ("MOM Visualizer");
  glutDisplayFunc (display);
}


/*--------------------------------------------------------------------------
   Init all what is initializable
  --------------------------------------------------------------------------*/
void drawModel (int argc, char **argv) {

  initWindow (argc, argv);

  // events...
  glutReshapeFunc  (reshape);
  glutKeyboardFunc (keyboard);
  glutMouseFunc    (mouse);
  glutMotionFunc   (motion);
  glutIdleFunc     (idle);
  glutTimerFunc    (1000, theTimer, 0);

  initView ();           // view...
  initRenderOptions ();  // render options...
  initMenu ();           // user interface: the menu

  glutMainLoop();        // the main loop
}


/*--------------------------------------------------------------------------
   Main program
  --------------------------------------------------------------------------*/
void main (int argc, char **argv) {

  if (argc!= 2)
    cout << "visMom file.mom" << endl;
  else {
    model.read (&argv[1][0]);
    NLods= model.getNLods();
    drawModel (argc, argv);
  }
}


/*--------------------------------------------------------------------------
   End of visMom.cpp
  --------------------------------------------------------------------------*/
